@@ -11,7 +11,6 @@
 #include "Accessible-inl.h"
 #include "nsAccUtils.h"
 #include "nsIAccessibleRelation.h"
-#include "nsIAccessibleText.h"
 #include "nsIAccessibleEditableText.h"
 #include "nsIPersistentProperties2.h"
 #include "Relation.h"
@@ -303,10 +302,10 @@ GetClosestInterestingAccessible(id anObject)
   // (which might be the owning NSWindow in the application, for example).
   //
   // get the native root accessible, and tell it to return its first parent unignored accessible.
-  RootAccessible* root = mGeckoAccessible->RootAccessible();
-  id nativeParent = GetNativeFromGeckoAccessible(static_cast<nsIAccessible*>(root));
+  id nativeParent =
+    GetNativeFromGeckoAccessible(mGeckoAccessible->RootAccessible());
   NSAssert1 (nativeParent, @"!!! we can't find a parent for %@", self);
-  
+
   return GetClosestInterestingAccessible(nativeParent);
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
@@ -393,12 +392,11 @@ GetClosestInterestingAccessible(id anObject)
   if (!mGeckoAccessible)
     return nil;
 
-  int32_t x = 0, y = 0, width = 0, height = 0;
-  mGeckoAccessible->GetBounds (&x, &y, &width, &height);
+  nsIntRect rect = mGeckoAccessible->Bounds();
   CGFloat scaleFactor =
     nsCocoaUtils::GetBackingScaleFactor([[NSScreen screens] objectAtIndex:0]);
-  return [NSValue valueWithSize:NSMakeSize(static_cast<CGFloat>(width) / scaleFactor,
-                                           static_cast<CGFloat>(height) / scaleFactor)];
+  return [NSValue valueWithSize:NSMakeSize(static_cast<CGFloat>(rect.width) / scaleFactor,
+                                           static_cast<CGFloat>(rect.height) / scaleFactor)];
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
@@ -410,7 +408,7 @@ GetClosestInterestingAccessible(id anObject)
 
 #ifdef DEBUG_A11Y
   NS_ASSERTION(nsAccUtils::IsTextInterfaceSupportCorrect(mGeckoAccessible),
-               "Does not support nsIAccessibleText when it should");
+               "Does not support Text when it should");
 #endif
 
 #define ROLE(geckoRole, stringRole, atkRole, macRole, msaaRole, ia2Role, nameRule) \
@@ -432,42 +430,37 @@ GetClosestInterestingAccessible(id anObject)
   if (!mGeckoAccessible)
     return nil;
 
-  // XXX maybe we should cache the subrole.
-  nsAutoString xmlRoles;
-
-  // XXX we don't need all the attributes (see bug 771113)
-  nsCOMPtr<nsIPersistentProperties> attributes = mGeckoAccessible->Attributes();
-  if (attributes)
-    nsAccUtils::GetAccAttr(attributes, nsGkAtoms::xmlroles, xmlRoles);
-
-  nsWhitespaceTokenizer tokenizer(xmlRoles);
-
-  while (tokenizer.hasMoreTokens()) {
-    const nsDependentSubstring token(tokenizer.nextToken());
-
-    if (token.EqualsLiteral("banner"))
+  nsIAtom* landmark = mGeckoAccessible->LandmarkRole();
+  if (landmark) {
+    if (landmark == nsGkAtoms::application)
+      return @"AXLandmarkApplication";
+    if (landmark == nsGkAtoms::banner)
       return @"AXLandmarkBanner";
-
-    if (token.EqualsLiteral("complementary"))
+    if (landmark == nsGkAtoms::complementary)
       return @"AXLandmarkComplementary";
-
-    if (token.EqualsLiteral("contentinfo"))
+    if (landmark == nsGkAtoms::contentinfo)
       return @"AXLandmarkContentInfo";
-
-    if (token.EqualsLiteral("main"))
+    if (landmark == nsGkAtoms::form)
+      return @"AXLandmarkForm";
+    if (landmark == nsGkAtoms::main)
       return @"AXLandmarkMain";
-
-    if (token.EqualsLiteral("navigation"))
+    if (landmark == nsGkAtoms::navigation)
       return @"AXLandmarkNavigation";
-
-    if (token.EqualsLiteral("search"))
+    if (landmark == nsGkAtoms::search)
       return @"AXLandmarkSearch";
+    if (landmark == nsGkAtoms::searchbox)
+      return @"AXSearchField";
   }
 
   switch (mRole) {
     case roles::LIST:
       return @"AXContentList"; // 10.6+ NSAccessibilityContentListSubrole;
 
+    case roles::ENTRY:
+      if (mGeckoAccessible->IsSearchbox())
+        return @"AXSearchField";
+      break;
+
     case roles::DEFINITION_LIST:
       return @"AXDefinitionList"; // 10.6+ NSAccessibilityDefinitionListSubrole;
 
@@ -477,6 +470,9 @@ GetClosestInterestingAccessible(id anObject)
     case roles::DEFINITION:
       return @"AXDefinition";
 
+    case roles::SWITCH:
+      return @"AXSwitch";
+
     default:
       break;
   }
@@ -484,6 +480,33 @@ GetClosestInterestingAccessible(id anObject)
   return nil;
 }
 
+struct RoleDescrMap
+{
+  NSString* role;
+  const nsString description;
+};
+
+static const RoleDescrMap sRoleDescrMap[] = {
+  { @"AXDefinition", NS_LITERAL_STRING("definition") },
+  { @"AXLandmarkBanner", NS_LITERAL_STRING("banner") },
+  { @"AXLandmarkComplementary", NS_LITERAL_STRING("complementary") },
+  { @"AXLandmarkContentInfo", NS_LITERAL_STRING("content") },
+  { @"AXLandmarkMain", NS_LITERAL_STRING("main") },
+  { @"AXLandmarkNavigation", NS_LITERAL_STRING("navigation") },
+  { @"AXLandmarkSearch", NS_LITERAL_STRING("search") },
+  { @"AXSearchField", NS_LITERAL_STRING("searchTextField") },
+  { @"AXTerm", NS_LITERAL_STRING("term") }
+};
+
+struct RoleDescrComparator
+{
+  const NSString* mRole;
+  explicit RoleDescrComparator(const NSString* aRole) : mRole(aRole) {}
+  int operator()(const RoleDescrMap& aEntry) const {
+    return [mRole compare:aEntry.role];
+  }
+};
+
 - (NSString*)roleDescription
 {
   if (mRole == roles::DOCUMENT)
@@ -491,30 +514,15 @@ GetClosestInterestingAccessible(id anObject)
 
   NSString* subrole = [self subrole];
 
-  if ((mRole == roles::LISTITEM) && [subrole isEqualToString:@"AXTerm"])
-    return utils::LocalizedString(NS_LITERAL_STRING("term"));
-  if ((mRole == roles::PARAGRAPH) && [subrole isEqualToString:@"AXDefinition"])
-    return utils::LocalizedString(NS_LITERAL_STRING("definition"));
-
-  NSString* role = [self role];
-
-  // the WAI-ARIA Landmarks
-  if ([role isEqualToString:NSAccessibilityGroupRole]) {
-    if ([subrole isEqualToString:@"AXLandmarkBanner"])
-      return utils::LocalizedString(NS_LITERAL_STRING("banner"));
-    if ([subrole isEqualToString:@"AXLandmarkComplementary"])
-      return utils::LocalizedString(NS_LITERAL_STRING("complementary"));
-    if ([subrole isEqualToString:@"AXLandmarkContentInfo"])
-      return utils::LocalizedString(NS_LITERAL_STRING("content"));
-    if ([subrole isEqualToString:@"AXLandmarkMain"])
-      return utils::LocalizedString(NS_LITERAL_STRING("main"));
-    if ([subrole isEqualToString:@"AXLandmarkNavigation"])
-      return utils::LocalizedString(NS_LITERAL_STRING("navigation"));
-    if ([subrole isEqualToString:@"AXLandmarkSearch"])
-      return utils::LocalizedString(NS_LITERAL_STRING("search"));
+  if (subrole) {
+    size_t idx = 0;
+    if (BinarySearchIf(sRoleDescrMap, 0, ArrayLength(sRoleDescrMap),
+                       RoleDescrComparator(subrole), &idx)) {
+      return utils::LocalizedString(sRoleDescrMap[idx].description);
+    }
   }
 
-  return NSAccessibilityRoleDescription(role, subrole);
+  return NSAccessibilityRoleDescription([self role], subrole);
 }
 
 - (NSString*)title
@@ -533,9 +541,8 @@ GetClosestInterestingAccessible(id anObject)
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
 
   nsAutoString value;
-  mGeckoAccessible->GetValue (value);
-  return value.IsEmpty() ? nil : [NSString stringWithCharacters:reinterpret_cast<const unichar*>(value.BeginReading())
-                                                         length:value.Length()];
+  mGeckoAccessible->Value(value);
+  return nsCocoaUtils::ToNSString(value);
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
@@ -578,9 +585,8 @@ GetClosestInterestingAccessible(id anObject)
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
 
   nsAutoString helpText;
-  mGeckoAccessible->GetHelp (helpText);
-  return helpText.IsEmpty() ? nil : [NSString stringWithCharacters:reinterpret_cast<const unichar*>(helpText.BeginReading())
-                                                            length:helpText.Length()];
+  mGeckoAccessible->Help(helpText);
+  return nsCocoaUtils::ToNSString(helpText);
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
