@@ -108,8 +108,14 @@ function remove(path, recursive) {
   }
 }
 
+/**
+ * Utility function to convert either an octal number or string
+ * into an octal number
+ * 0777 => 0777
+ * "0644" => 0644
+ */
 function Mode(mode, fallback) {
-  return isString(mode) ? parseInt(mode) : mode || fallback;
+  return isString(mode) ? parseInt(mode, 8) : mode || fallback;
 }
 function Flags(flag) {
   return !isString(flag) ? flag :
@@ -394,8 +400,15 @@ exports.lchown = lchown;
  * Synchronous chmod(2).
  */
 function chmodSync (path, mode) {
-  throw Error("Not implemented yet!!");
-};
+  let file;
+  try {
+    file = new nsILocalFile(path);
+  } catch(e) {
+    throw FSError("chmod", "ENOENT", 34, path);
+  }
+
+  file.permissions = Mode(mode);
+}
 exports.chmodSync = chmodSync;
 /**
  * Asynchronous chmod(2). No arguments other than a possible exception are
@@ -416,7 +429,7 @@ exports.fchmodSync = fchmodSync;
  * given to the completion callback.
  */
 let fchmod = Async(fchmodSync);
-exports.chmod = fchmod;
+exports.fchmod = fchmod;
 
 
 /**
@@ -634,16 +647,16 @@ exports.close = close;
 /**
  * Synchronous open(2).
  */
-function openSync(path, flags, mode) {
+function openSync(aPath, aFlag, aMode) {
   let [ fd, flags, mode, file ] =
-      [ { path: path }, Flags(flags), Mode(mode), nsILocalFile(path) ];
+      [ { path: aPath }, Flags(aFlag), Mode(aMode), nsILocalFile(aPath) ];
 
   nsIFile(fd, file);
 
   // If trying to open file for just read that does not exists
   // need to throw exception as node does.
   if (!file.exists() && !isWritable(flags))
-    throw FSError("open", "ENOENT", 34, path);
+    throw FSError("open", "ENOENT", 34, aPath);
 
   // If we want to open file in read mode we initialize input stream.
   if (isReadable(flags)) {
@@ -679,8 +692,6 @@ function writeSync(fd, buffer, offset, length, position) {
   else if (length + offset !== buffer.length) {
     buffer = buffer.slice(offset, offset + length);
   }
-  let writeStream = new WriteStream(fd, { position: position,
-                                          length: length });
 
   let output = BinaryOutputStream(nsIFileOutputStream(fd));
   nsIBinaryOutputStream(fd, output);
@@ -798,12 +809,7 @@ function readFile(path, encoding, callback) {
     let readStream = new ReadStream(path);
     readStream.on("data", function(data) {
       if (!buffer) buffer = data;
-      else {
-        let bufferred = buffer
-        buffer = new Buffer(buffer.length + data.length);
-        bufferred.copy(buffer, 0);
-        data.copy(buffer, bufferred.length);
-      }
+      else buffer = Buffer.concat([buffer, data], 2);
     });
     readStream.on("error", function onError(error) {
       callback(error);
@@ -814,7 +820,8 @@ function readFile(path, encoding, callback) {
       readStream.destroy();
       callback(null, buffer);
     });
-  } catch (error) {
+  }
+  catch (error) {
     setTimeout(callback, 0, error);
   }
 };
@@ -844,6 +851,9 @@ exports.readFileSync = readFileSync;
  * exists. data can be a string or a buffer.
  */
 function writeFile(path, content, encoding, callback) {
+  if (!isString(path))
+    throw new TypeError('path must be a string');
+
   try {
     if (isFunction(encoding)) {
       callback = encoding
@@ -874,6 +884,7 @@ exports.writeFile = writeFile;
  * The synchronous version of `fs.writeFile`.
  */
 function writeFileSync(filename, data, encoding) {
+  // TODO: Implement this in bug 1148209 https://bugzilla.mozilla.org/show_bug.cgi?id=1148209
   throw Error("Not implemented");
 };
 exports.writeFileSync = writeFileSync;
