@@ -22,6 +22,10 @@ const Cu = Components.utils;
 Cu.import('resource://gre/modules/Services.jsm');
 Cu.import('resource://gre/modules/XPCOMUtils.jsm');
 
+XPCOMUtils.defineLazyServiceGetter(this, "settings",
+                                   "@mozilla.org/settingsService;1",
+                                   "nsISettingsService");
+
 function debug(msg) {
   log(msg);
 }
@@ -89,14 +93,23 @@ ProcessGlobal.prototype = {
     }
   },
 
-  processWipeFile: function(text) {
-    log("processWipeFile " + text);
+  processCommandsFile: function(text) {
+    log("processCommandsFile " + text);
     let lines = text.split("\n");
     lines.forEach((line) => {
       log(line);
       let params = line.split(" ");
       if (params[0] == "wipe") {
         this.wipeDir(params[1]);
+      } else if (params[0] == "root") {
+        log("unrestrict devtools");
+        Services.prefs.setBoolPref("devtools.debugger.forbid-certified-apps", false);
+        Services.prefs.setBoolPref("dom.apps.developer_mode", true);
+        // TODO: Remove once bug 1125916 is fixed.
+        Services.prefs.setBoolPref("network.disable.ipc.security", true);
+        Services.prefs.setBoolPref("dom.webcomponents.enabled", true);
+        let lock = settings.createLock();
+        lock.set("developer.menu.enabled", true, null);
       }
     });
   },
@@ -113,7 +126,7 @@ ProcessGlobal.prototype = {
     let file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsIFile);
     file.initWithPath(postResetFile);
     if (!file.exists()) {
-      debug("Nothing to wipe.")
+      debug("No additional command.")
       return;
     }
 
@@ -122,7 +135,7 @@ ProcessGlobal.prototype = {
       (array) => {
         file.remove(false);
         let decoder = new TextDecoder();
-        this.processWipeFile(decoder.decode(array));
+        this.processCommandsFile(decoder.decode(array));
       },
       function onError(error) {
         debug("Error: " + error);
@@ -157,7 +170,8 @@ ProcessGlobal.prototype = {
       let args = message.arguments;
       let stackTrace = '';
 
-      if (message.level == 'assert' || message.level == 'error' || message.level == 'trace') {
+      if (message.stacktrace &&
+          (message.level == 'assert' || message.level == 'error' || message.level == 'trace')) {
         stackTrace = Array.map(message.stacktrace, formatStackFrame).join('\n');
       } else {
         stackTrace = formatStackFrame(message);
