@@ -17,6 +17,8 @@ XPCOMUtils.defineLazyModuleGetter(this, "Task",
                                   "resource://gre/modules/Task.jsm");
 XPCOMUtils.defineLazyModuleGetter(this, "DownloadsCommon",
                                   "resource:///modules/DownloadsCommon.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "TelemetryStopwatch",
+                                  "resource://gre/modules/TelemetryStopwatch.jsm");
 
 function Sanitizer() {}
 Sanitizer.prototype = {
@@ -92,6 +94,8 @@ Sanitizer.prototype = {
       return deferred.promise;
     }
 
+    TelemetryStopwatch.start("FX_SANITIZE_TOTAL");
+
     // Cache the range of times to clear
     if (this.ignoreTimespan)
       var range = null;  // If we ignore timespan, clear everything
@@ -101,6 +105,7 @@ Sanitizer.prototype = {
     let itemCount = Object.keys(itemsToClear).length;
     let onItemComplete = function() {
       if (!--itemCount) {
+        TelemetryStopwatch.finish("FX_SANITIZE_TOTAL");
         seenError ? deferred.reject() : deferred.resolve();
       }
     };
@@ -146,6 +151,8 @@ Sanitizer.prototype = {
     cache: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_CACHE");
+
         var cache = Cc["@mozilla.org/netwerk/cache-storage-service;1"].
                     getService(Ci.nsICacheStorageService);
         try {
@@ -159,6 +166,8 @@ Sanitizer.prototype = {
         try {
           imageCache.clearCache(false); // true=chrome, false=content
         } catch(er) {}
+
+        TelemetryStopwatch.finish("FX_SANITIZE_CACHE");
       },
 
       get canClear()
@@ -170,6 +179,8 @@ Sanitizer.prototype = {
     cookies: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_COOKIES");
+
         var cookieMgr = Components.classes["@mozilla.org/cookiemanager;1"]
                                   .getService(Ci.nsICookieManager);
         if (this.range) {
@@ -188,6 +199,11 @@ Sanitizer.prototype = {
           cookieMgr.removeAll();
         }
 
+        // Clear deviceIds. Done asynchronously (returns before complete).
+        let mediaMgr = Components.classes["@mozilla.org/mediaManagerService;1"]
+                                 .getService(Ci.nsIMediaManagerService);
+        mediaMgr.sanitizeDeviceIds(this.range && this.range[0]);
+
         // Clear plugin data.
         const phInterface = Ci.nsIPluginHost;
         const FLAG_CLEAR_ALL = phInterface.FLAG_CLEAR_ALL;
@@ -217,6 +233,8 @@ Sanitizer.prototype = {
             }
           }
         }
+
+        TelemetryStopwatch.finish("FX_SANITIZE_COOKIES");
       },
 
       get canClear()
@@ -228,8 +246,10 @@ Sanitizer.prototype = {
     offlineApps: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_OFFLINEAPPS");
         Components.utils.import("resource:///modules/offlineAppCache.jsm");
         OfflineAppCacheHelper.clear();
+        TelemetryStopwatch.finish("FX_SANITIZE_OFFLINEAPPS");
       },
 
       get canClear()
@@ -241,6 +261,8 @@ Sanitizer.prototype = {
     history: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_HISTORY");
+
         if (this.range)
           PlacesUtils.history.removeVisitsByTimeframe(this.range[0], this.range[1]);
         else
@@ -249,7 +271,8 @@ Sanitizer.prototype = {
         try {
           var os = Components.classes["@mozilla.org/observer-service;1"]
                              .getService(Components.interfaces.nsIObserverService);
-          os.notifyObservers(null, "browser:purge-session-history", "");
+          let clearStartingTime = this.range ? String(this.range[0]) : "";
+          os.notifyObservers(null, "browser:purge-session-history", clearStartingTime);
         }
         catch (e) { }
 
@@ -258,6 +281,8 @@ Sanitizer.prototype = {
                                     .getService(Components.interfaces.nsINetworkPredictor);
           predictor.reset();
         } catch (e) { }
+
+        TelemetryStopwatch.finish("FX_SANITIZE_HISTORY");
       },
 
       get canClear()
@@ -271,6 +296,8 @@ Sanitizer.prototype = {
     formdata: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_FORMDATA");
+
         // Clear undo history of all searchBars
         var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1']
                                       .getService(Components.interfaces.nsIWindowMediator);
@@ -295,6 +322,8 @@ Sanitizer.prototype = {
           [ change.firstUsedStart, change.firstUsedEnd ] = this.range;
         }
         FormHistory.update(change);
+
+        TelemetryStopwatch.finish("FX_SANITIZE_FORMDATA");
       },
 
       canClear : function(aCallback, aArg)
@@ -341,6 +370,7 @@ Sanitizer.prototype = {
     downloads: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_DOWNLOADS");
         Task.spawn(function () {
           let filterByTime = null;
           if (this.range) {
@@ -354,7 +384,11 @@ Sanitizer.prototype = {
           // Clear all completed/cancelled downloads
           let list = yield Downloads.getList(Downloads.ALL);
           list.removeFinished(filterByTime);
-        }.bind(this)).then(null, Components.utils.reportError);
+          TelemetryStopwatch.finish("FX_SANITIZE_DOWNLOADS");
+        }.bind(this)).then(null, error => {
+          TelemetryStopwatch.finish("FX_SANITIZE_DOWNLOADS");
+          Components.utils.reportError(error);
+        });
       },
 
       canClear : function(aCallback, aArg)
@@ -367,10 +401,12 @@ Sanitizer.prototype = {
     passwords: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_PASSWORDS");
         var pwmgr = Components.classes["@mozilla.org/login-manager;1"]
                               .getService(Components.interfaces.nsILoginManager);
         // Passwords are timeless, and don't respect the timeSpan setting
         pwmgr.removeAllLogins();
+        TelemetryStopwatch.finish("FX_SANITIZE_PASSWORDS");
       },
 
       get canClear()
@@ -385,6 +421,8 @@ Sanitizer.prototype = {
     sessions: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_SESSIONS");
+
         // clear all auth tokens
         var sdr = Components.classes["@mozilla.org/security/sdr;1"]
                             .getService(Components.interfaces.nsISecretDecoderRing);
@@ -394,6 +432,8 @@ Sanitizer.prototype = {
         var os = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
         os.notifyObservers(null, "net:clear-active-logins", null);
+
+        TelemetryStopwatch.finish("FX_SANITIZE_SESSIONS");
       },
 
       get canClear()
@@ -405,6 +445,8 @@ Sanitizer.prototype = {
     siteSettings: {
       clear: function ()
       {
+        TelemetryStopwatch.start("FX_SANITIZE_SITESETTINGS");
+
         // Clear site-specific permissions like "Allow this site to open popups"
         // we ignore the "end" range and hope it is now() - none of the
         // interfaces used here support a true range anyway.
@@ -443,6 +485,13 @@ Sanitizer.prototype = {
         var sss = Cc["@mozilla.org/ssservice;1"]
                     .getService(Ci.nsISiteSecurityService);
         sss.clearAll();
+
+        // Clear all push notification subscriptions
+        var push = Cc["@mozilla.org/push/NotificationService;1"]
+                    .getService(Ci.nsIPushNotificationService);
+        push.clearAll();
+
+        TelemetryStopwatch.finish("FX_SANITIZE_SITESETTINGS");
       },
 
       get canClear()
@@ -515,6 +564,8 @@ Sanitizer.prototype = {
 
         // If/once we get here, we should actually be able to close all windows.
 
+        TelemetryStopwatch.start("FX_SANITIZE_OPENWINDOWS");
+
         // First create a new window. We do this first so that on non-mac, we don't
         // accidentally close the app by closing all the windows.
         let handler = Cc["@mozilla.org/browser/clh;1"].getService(Ci.nsIBrowserHandler);
@@ -554,8 +605,10 @@ Sanitizer.prototype = {
 #endif
           newWindowOpened = true;
           // If we're the last thing to happen, invoke callback.
-          if (numWindowsClosing == 0)
+          if (numWindowsClosing == 0) {
+            TelemetryStopwatch.finish("FX_SANITIZE_OPENWINDOWS");
             aCallback();
+          }
         }
 
         let numWindowsClosing = windowList.length;
@@ -564,8 +617,10 @@ Sanitizer.prototype = {
           if (numWindowsClosing == 0) {
             Services.obs.removeObserver(onWindowClosed, "xul-window-destroyed");
             // If we're the last thing to happen, invoke callback.
-            if (newWindowOpened)
+            if (newWindowOpened) {
+              TelemetryStopwatch.finish("FX_SANITIZE_OPENWINDOWS");
               aCallback();
+            }
           }
         }
 
