@@ -1,5 +1,3 @@
-
-
 let AddonManager = Cu.import("resource://gre/modules/AddonManager.jsm", {}).AddonManager;
 let SocialService = Cu.import("resource://gre/modules/SocialService.jsm", {}).SocialService;
 
@@ -84,6 +82,25 @@ function installListener(next, aManifest) {
 }
 
 var tests = {
+  testHTTPInstallFailure: function(next) {
+    let activationURL = "http://example.com/browser/browser/base/content/test/social/social_activate.html"
+    addTab(activationURL, function(tab) {
+      let doc = tab.linkedBrowser.contentDocument;
+      let installFrom = doc.nodePrincipal.origin;
+      is(SocialService.getOriginActivationType(installFrom), "foreign", "testing foriegn install");
+      let data = {
+        origin: doc.nodePrincipal.origin,
+        url: doc.location.href,
+        manifest: manifest,
+        window: window
+      }
+      Social.installProvider(data, function(addonManifest) {
+        ok(!addonManifest, "unable to install provider over http");
+        gBrowser.removeTab(tab);
+        next();
+      });
+    });
+  },
   testAddonEnableToggle: function(next) {
     let expectEvent;
     let prefname = getManifestPrefname(manifest);
@@ -193,7 +210,13 @@ var tests = {
       let installFrom = doc.nodePrincipal.origin;
       Services.prefs.setCharPref("social.whitelist", "");
       is(SocialService.getOriginActivationType(installFrom), "foreign", "testing foriegn install");
-      Social.installProvider(doc, manifest2, function(addonManifest) {
+      let data = {
+        origin: doc.nodePrincipal.origin,
+        url: doc.location.href,
+        manifest: manifest2,
+        window: window
+      }
+      Social.installProvider(data, function(addonManifest) {
         Services.prefs.clearUserPref("social.whitelist");
         SocialService.enableProvider(addonManifest.origin, function(provider) {
           Social.uninstallProvider(addonManifest.origin);
@@ -217,7 +240,13 @@ var tests = {
       let installFrom = doc.nodePrincipal.origin;
       Services.prefs.setCharPref("social.directories", installFrom);
       is(SocialService.getOriginActivationType(installFrom), "directory", "testing directory install");
-      Social.installProvider(doc, manifest2, function(addonManifest) {
+      let data = {
+        origin: installFrom,
+        url: doc.location.href,
+        manifest: manifest2,
+        window: window
+      }
+      Social.installProvider(data, function(addonManifest) {
         Services.prefs.clearUserPref("social.directories");
         SocialService.enableProvider(addonManifest.origin, function(provider) {
           Social.uninstallProvider(addonManifest.origin);
@@ -241,7 +270,13 @@ var tests = {
       let doc = tab.linkedBrowser.contentDocument;
       let installFrom = doc.nodePrincipal.origin;
       Services.prefs.setCharPref("social.whitelist", installFrom);
-      Social.installProvider(doc, manifest2, function(addonManifest) {
+      let data = {
+        origin: installFrom,
+        url: doc.location.href,
+        manifest: manifest2,
+        window: window
+      }
+      Social.installProvider(data, function(addonManifest) {
         SocialService.enableProvider(addonManifest.origin, function(provider) {
           is(provider.manifest.version, 1, "manifest version is 1");
 
