@@ -2,7 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-pref("startup.homepage_override_url","https://www.mozilla.org/%LOCALE%/firefox/%VERSION%/whatsnew/?oldversion=%OLD_VERSION%");
+pref("startup.homepage_override_url","");
 pref("startup.homepage_welcome_url","https://www.mozilla.org/%LOCALE%/firefox/%VERSION%/firstrun/");
 // Interval: Time between checks for a new version (in seconds)
 pref("app.update.interval", 43200); // 12 hours
@@ -26,7 +26,7 @@ pref("app.update.checkInstallTime.days", 63);
 // code usage depends on contracts, please contact the Firefox module owner if you have questions
 pref("browser.search.param.yahoo-fr", "moz35");
 pref("browser.search.param.yahoo-fr-ja", "mozff");
-#ifdef MOZ_METRO
-pref("browser.search.param.ms-pc-metro", "MOZW");
-pref("browser.search.param.yahoo-fr-metro", "mozilla_metro_search");
-#endif
+
+// Number of usages of the web console or scratchpad.
+// If this is less than 5, then pasting code into the web console or scratchpad is disabled
+pref("devtools.selfxss.count", 0);
\ No newline at end of file
