@@ -16,8 +16,7 @@ loop.conversation = (function(mozL10n) {
   var sharedModels = loop.shared.models;
   var sharedActions = loop.shared.actions;
 
-  var IncomingConversationView = loop.conversationViews.IncomingConversationView;
-  var OutgoingConversationView = loop.conversationViews.OutgoingConversationView;
+  var CallControllerView = loop.conversationViews.CallControllerView;
   var CallIdentifierView = loop.conversationViews.CallIdentifierView;
   var DesktopRoomConversationView = loop.roomViews.DesktopRoomConversationView;
   var GenericFailureView = loop.conversationViews.GenericFailureView;
@@ -27,65 +26,36 @@ loop.conversation = (function(mozL10n) {
    * in progress, and hence, which view to display.
    */
   var AppControllerView = React.createClass({
-    mixins: [Backbone.Events, sharedMixins.WindowCloseMixin],
+    mixins: [
+      Backbone.Events,
+      loop.store.StoreMixin("conversationAppStore"),
+      sharedMixins.WindowCloseMixin
+    ],
 
     propTypes: {
-      // XXX Old types required for incoming call view.
-      client: React.PropTypes.instanceOf(loop.Client).isRequired,
-      conversation: React.PropTypes.instanceOf(sharedModels.ConversationModel)
-                         .isRequired,
-      sdk: React.PropTypes.object.isRequired,
-
-      // XXX New types for flux style
-      conversationAppStore: React.PropTypes.instanceOf(
-        loop.store.ConversationAppStore).isRequired,
-      conversationStore: React.PropTypes.instanceOf(loop.store.ConversationStore)
-                              .isRequired,
       dispatcher: React.PropTypes.instanceOf(loop.Dispatcher).isRequired,
       roomStore: React.PropTypes.instanceOf(loop.store.RoomStore),
-      feedbackStore:
-        React.PropTypes.instanceOf(loop.store.FeedbackStore).isRequired
+      mozLoop: React.PropTypes.object.isRequired
     },
 
     getInitialState: function() {
-      return this.props.conversationAppStore.getStoreState();
-    },
-
-    componentWillMount: function() {
-      this.listenTo(this.props.conversationAppStore, "change", function() {
-        this.setState(this.props.conversationAppStore.getStoreState());
-      }, this);
-    },
-
-    componentWillUnmount: function() {
-      this.stopListening(this.props.conversationAppStore);
+      return this.getStoreState();
     },
 
     render: function() {
       switch(this.state.windowType) {
-        case "incoming": {
-          return (<IncomingConversationView
-            client={this.props.client}
-            conversation={this.props.conversation}
-            sdk={this.props.sdk}
-            isDesktop={true}
-            conversationAppStore={this.props.conversationAppStore}
-            feedbackStore={this.props.feedbackStore}
-          />);
-        }
+        // CallControllerView is used for both.
+        case "incoming":
         case "outgoing": {
-          return (<OutgoingConversationView
-            store={this.props.conversationStore}
+          return (<CallControllerView
             dispatcher={this.props.dispatcher}
-            feedbackStore={this.props.feedbackStore}
-          />);
+            mozLoop={this.props.mozLoop} />);
         }
         case "room": {
           return (<DesktopRoomConversationView
             dispatcher={this.props.dispatcher}
-            roomStore={this.props.roomStore}
-            feedbackStore={this.props.feedbackStore}
-          />);
+            mozLoop={this.props.mozLoop}
+            roomStore={this.props.roomStore} />);
         }
         case "failed": {
           return <GenericFailureView cancelCall={this.closeWindow} />;
@@ -126,8 +96,13 @@ loop.conversation = (function(mozL10n) {
     var sdkDriver = new loop.OTSdkDriver({
       isDesktop: true,
       dispatcher: dispatcher,
-      sdk: OT
+      sdk: OT,
+      mozLoop: navigator.mozLoop
     });
+
+    // expose for functional tests
+    loop.conversation._sdkDriver = sdkDriver;
+
     var appVersionInfo = navigator.mozLoop.appVersionInfo;
     var feedbackClient = new loop.FeedbackAPIClient(
       navigator.mozLoop.getLoopPref("feedback.baseUrl"), {
@@ -161,16 +136,14 @@ loop.conversation = (function(mozL10n) {
       feedbackClient: feedbackClient
     });
 
-    // XXX Old class creation for the incoming conversation view, whilst
-    // we transition across (bug 1072323).
-    var conversation = new sharedModels.ConversationModel({}, {
-      sdk: window.OT,
-      mozLoop: navigator.mozLoop
+    loop.store.StoreMixin.register({
+      conversationAppStore: conversationAppStore,
+      conversationStore: conversationStore,
+      feedbackStore: feedbackStore
     });
 
     // Obtain the windowId and pass it through
-    var helper = new loop.shared.utils.Helper();
-    var locationHash = helper.locationData().hash;
+    var locationHash = loop.shared.utils.locationData().hash;
     var windowId;
 
     var hash = locationHash.match(/#(.*)/);
@@ -178,28 +151,19 @@ loop.conversation = (function(mozL10n) {
       windowId = hash[1];
     }
 
-    conversation.set({windowId: windowId});
-
     window.addEventListener("unload", function(event) {
-      // Handle direct close of dialog box via [x] control.
-      // XXX Move to the conversation models, when we transition
-      // incoming calls to flux (bug 1088672).
-      navigator.mozLoop.calls.clearCallInProgress(windowId);
-
       dispatcher.dispatch(new sharedActions.WindowUnload());
     });
 
     React.render(<AppControllerView
-      conversationAppStore={conversationAppStore}
       roomStore={roomStore}
-      feedbackStore={feedbackStore}
-      conversationStore={conversationStore}
-      client={client}
-      conversation={conversation}
       dispatcher={dispatcher}
-      sdk={window.OT}
+      mozLoop={navigator.mozLoop}
     />, document.querySelector('#main'));
 
+    document.body.setAttribute("dir", mozL10n.getDirection());
+    document.body.setAttribute("platform", loop.shared.utils.getPlatform());
+
     dispatcher.dispatch(new sharedActions.GetWindowData({
       windowId: windowId
     }));
@@ -207,7 +171,15 @@ loop.conversation = (function(mozL10n) {
 
   return {
     AppControllerView: AppControllerView,
-    init: init
+    init: init,
+
+    /**
+     * Exposed for the use of functional tests to be able to check
+     * metric-related execution as the call sequence progresses.
+     *
+     * @type loop.OTSdkDriver
+     */
+    _sdkDriver: null
   };
 })(document.mozL10n);
 
