@@ -1,10 +1,13 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
+"use strict";
+
 const HAWK_TOKEN_LENGTH = 64;
 const {
   LOOP_SESSION_TYPE,
   MozLoopServiceInternal,
+  MozLoopService,
 } = Cu.import("resource:///modules/loop/MozLoopService.jsm", {});
 const {LoopCalls} = Cu.import("resource:///modules/loop/LoopCalls.jsm", {});
 const {LoopRooms} = Cu.import("resource:///modules/loop/LoopRooms.jsm", {});
@@ -14,12 +17,13 @@ const {LoopRooms} = Cu.import("resource:///modules/loop/LoopRooms.jsm", {});
 // if offline mode is requested multiple times in a test run.
 const WAS_OFFLINE = Services.io.offline;
 
+
 var gMozLoopAPI;
 
 function promiseGetMozLoopAPI() {
   return new Promise((resolve, reject) => {
     let loopPanel = document.getElementById("loop-notification-panel");
-    let btn = document.getElementById("loop-button-throttled");
+    let btn = document.getElementById("loop-button");
 
     // Wait for the popup to be shown if it's not already, then we can get the iframe and
     // wait for the iframe's load to be completed.
@@ -70,6 +74,34 @@ function promiseGetMozLoopAPI() {
   });
 }
 
+function waitForCondition(condition, nextTest, errorMsg) {
+  var tries = 0;
+  var interval = setInterval(function() {
+    if (tries >= 30) {
+      ok(false, errorMsg);
+      moveOn();
+    }
+    var conditionPassed;
+    try {
+      conditionPassed = condition();
+    } catch (e) {
+      ok(false, e + "\n" + e.stack);
+      conditionPassed = false;
+    }
+    if (conditionPassed) {
+      moveOn();
+    }
+    tries++;
+  }, 100);
+  var moveOn = function() { clearInterval(interval); nextTest(); };
+}
+
+function promiseWaitForCondition(aConditionFn) {
+  let deferred = Promise.defer();
+  waitForCondition(aConditionFn, deferred.resolve, "Condition didn't pass.");
+  return deferred.promise;
+}
+
 /**
  * Loads the loop panel by clicking the button and waits for its open to complete.
  * It also registers
@@ -93,8 +125,8 @@ function loadLoopPanel(aOverrideOptions = {}) {
   let loopPanel = document.getElementById("loop-notification-panel");
   loopPanel.setAttribute("animate", "false");
 
-  // Now get the actual API.
-  yield promiseGetMozLoopAPI();
+  // Now get the actual API loaded into gMozLoopAPI.
+  return promiseGetMozLoopAPI();
 }
 
 function promiseOAuthParamsSetup(baseURL, params) {
@@ -114,7 +146,6 @@ function* resetFxA() {
   global.gHawkClient = null;
   global.gFxAOAuthClientPromise = null;
   global.gFxAOAuthClient = null;
-  MozLoopServiceInternal.deferredRegistrations.delete(LOOP_SESSION_TYPE.FXA);
   MozLoopServiceInternal.fxAOAuthProfile = null;
   MozLoopServiceInternal.fxAOAuthTokenData = null;
   const fxASessionPref = MozLoopServiceInternal.getSessionTokenPrefName(LOOP_SESSION_TYPE.FXA);
@@ -176,6 +207,50 @@ function promiseOAuthGetRegistration(baseURL) {
   });
 }
 
+/**
+ * Waits for a load (or custom) event to finish in a given tab. If provided
+ * load an uri into the tab.
+ *
+ * @param tab
+ *        The tab to load into.
+ * @param [optional] url
+ *        The url to load, or the current url.
+ * @param [optional] event
+ *        The load event type to wait for.  Defaults to "load".
+ * @return {Promise} resolved when the event is handled.
+ * @resolves to the received event
+ * @rejects if a valid load event is not received within a meaningful interval
+ */
+function promiseTabLoadEvent(tab, url, eventType="load") {
+  return new Promise((resolve, reject) => {
+    info("Wait tab event: " + eventType);
+
+    function handle(event) {
+      if (event.originalTarget != tab.linkedBrowser.contentDocument ||
+          event.target.location.href == "about:blank" ||
+          (url && event.target.location.href != url)) {
+        info("Skipping spurious '" + eventType + "'' event" +
+             " for " + event.target.location.href);
+        return;
+      }
+      clearTimeout(timeout);
+      tab.linkedBrowser.removeEventListener(eventType, handle, true);
+      info("Tab event received: " + eventType);
+      resolve(event);
+    }
+
+    let timeout = setTimeout(() => {
+      if (tab.linkedBrowser)
+        tab.linkedBrowser.removeEventListener(eventType, handle, true);
+      reject(new Error("Timed out while waiting for a '" + eventType + "'' event"));
+    }, 30000);
+
+    tab.linkedBrowser.addEventListener(eventType, handle, true, true);
+    if (url)
+      tab.linkedBrowser.loadURI(url);
+  });
+}
+
 function getLoopString(stringID) {
   return MozLoopServiceInternal.localizedStrings.get(stringID);
 }
@@ -189,7 +264,7 @@ let mockPushHandler = {
   // This sets the registration result to be returned when initialize
   // is called. By default, it is equivalent to success.
   registrationResult: null,
-  registrationPushURL: null,
+  registrationPushURLs: {},
   notificationCallback: {},
   registeredChannels: {},
 
@@ -200,12 +275,24 @@ let mockPushHandler = {
     if ("mockWebSocket" in options) {
       this._mockWebSocket = options.mockWebSocket;
     }
+    this.registrationPushURLs[MozLoopService.channelIDs.callsGuest] =
+      "https://localhost/pushUrl/guest-calls";
+    this.registrationPushURLs[MozLoopService.channelIDs.roomsGuest] =
+      "https://localhost/pushUrl/guest-rooms";
+    this.registrationPushURLs[MozLoopService.channelIDs.callsFxA] =
+      "https://localhost/pushUrl/fxa-calls";
+    this.registrationPushURLs[MozLoopService.channelIDs.roomsFxA] =
+      "https://localhost/pushUrl/fxa-rooms";
   },
 
   register: function(channelId, registerCallback, notificationCallback) {
     this.notificationCallback[channelId] = notificationCallback;
-    this.registeredChannels[channelId] = this.registrationPushURL;
-    setTimeout(registerCallback(this.registrationResult, this.registrationPushURL, channelId), 0);
+    this.registeredChannels[channelId] = this.registrationPushURLs[channelId];
+    setTimeout(registerCallback(this.registrationResult, this.registeredChannels[channelId], channelId), 0);
+  },
+
+  unregister: function(channelID) {
+    return;
   },
 
   /**
@@ -216,13 +303,6 @@ let mockPushHandler = {
   }
 };
 
-// Add the Loop button to the navbar.
-CustomizableUI.addWidgetToArea("loop-button-throttled", CustomizableUI.AREA_NAVBAR);
-
-registerCleanupFunction(function() {
-  CustomizableUI.reset();
-});
-
 const mockDb = {
   _store: { },
   _next_guid: 1,
@@ -241,7 +321,7 @@ const mockDb = {
     callback(null, details);
   },
   remove: function(guid, callback) {
-    if (!guid in this._store) {
+    if (!(guid in this._store)) {
       callback(new Error("Could not find _guid '" + guid + "' in database"));
       return;
     }
