@@ -7,41 +7,45 @@
  *   Raymond Lee <raymond@appcoast.com>
  */
 
+"use strict";
+
 const TEST_URL = 'data:text/html,<script>window.onbeforeunload=' +
-                 'function(e){e.returnValue="?"}</script><body ' +
-                 'onunload="alert(\'onunload\')" onpagehide="' +
-                 'alert(\'onpagehide\')"></body>';
+                 'function(e){e.returnValue="?"}</script>';
 
 let contentWindow;
 let activeGroup;
 
+Components.utils.import("resource://gre/modules/Promise.jsm", this);
+
 function test() {
   waitForExplicitFinish();
 
-  showTabView(function () {
-    contentWindow = TabView.getContentWindow();
+  newWindowWithTabView(win => {
+    contentWindow = win.TabView.getContentWindow();
     activeGroup = contentWindow.GroupItems.getActiveGroupItem();
 
-    gBrowser.browsers[0].loadURI("data:text/html,<p>test for bug 626455, tab1");
-    gBrowser.addTab(TEST_URL);
+    win.gBrowser.browsers[0].loadURI("data:text/html,<p>test for bug 626455, tab1");
 
-    afterAllTabsLoaded(testStayOnPage);
+    let tab = win.gBrowser.addTab(TEST_URL);
+    afterAllTabsLoaded(() => testStayOnPage(win, tab));
   });
 }
 
-function testStayOnPage() {
-  whenDialogOpened(function (dialog) {
+function testStayOnPage(win, blockingTab) {
+  let browser = blockingTab.linkedBrowser;
+  waitForOnBeforeUnloadDialog(browser, function (btnLeave, btnStay) {
     // stay on page
-    dialog.cancelDialog();
+    btnStay.click();
 
     executeSoon(function () {
       showTabView(function () {
-        is(gBrowser.tabs.length, 1,
+        is(win.gBrowser.tabs.length, 1,
            "The total number of tab is 1 when staying on the page");
 
-        let location = gBrowser.browsers[0].currentURI.spec;
-        isnot(location.indexOf("onbeforeunload"), -1,
-              "The open tab is the expected one");
+        // The other initial tab has been closed when trying to close the tab
+        // group. The only tab left is the one with the onbeforeunload dialog.
+        let url = win.gBrowser.browsers[0].currentURI.spec;
+        ok(url.includes("onbeforeunload"), "The open tab is the expected one");
 
         is(contentWindow.GroupItems.getActiveGroupItem(), activeGroup,
            "Active group is still the same");
@@ -50,33 +54,32 @@ function testStayOnPage() {
            "Only one group is open");
 
         // start the next test
-        testLeavePage();
-      });
+        testLeavePage(win, win.gBrowser.tabs[0]);
+      }, win);
     });
   });
 
   closeGroupItem(activeGroup);
 }
 
-function testLeavePage() {
-  let dialogsAccepted = 0;
-
-  whenDialogOpened(function onDialogOpened(dialog) {
+function testLeavePage(win, blockingTab) {
+  let browser = blockingTab.linkedBrowser;
+  waitForOnBeforeUnloadDialog(browser, function (btnLeave, btnStay) {
     // Leave page
-    dialog.acceptDialog();
+    btnLeave.click();
   });
 
-  whenGroupClosed(activeGroup, finishTest);
+  whenGroupClosed(activeGroup, () => finishTest(win));
   closeGroupItem(activeGroup);
 }
 
-function finishTest() {
-  is(gBrowser.tabs.length, 1,
+function finishTest(win) {
+  is(win.gBrowser.tabs.length, 1,
      "The total number of tab is 1 after leaving the page");
   is(contentWindow.TabItems.getItems().length, 1,
      "The total number of tab items is 1 after leaving the page");
 
-  let location = gBrowser.browsers[0].currentURI.spec;
+  let location = win.gBrowser.browsers[0].currentURI.spec;
   is(location, BROWSER_NEW_TAB_URL, "The open tab is the expected one");
 
   isnot(contentWindow.GroupItems.getActiveGroupItem(), activeGroup,
@@ -87,7 +90,7 @@ function finishTest() {
 
   contentWindow = null;
   activeGroup = null;
-  finish();
+  promiseWindowClosed(win).then(finish);
 }
 
 // ----------
@@ -97,29 +100,3 @@ function whenGroupClosed(group, callback) {
     callback();
   });
 }
-
-// ----------
-function whenDialogOpened(callback) {
-  let wm = Cc["@mozilla.org/appshell/window-mediator;1"]
-           .getService(Ci.nsIWindowMediator);
-
-  let listener = {
-    onCloseWindow: function () {},
-    onWindowTitleChange: function () {},
-
-    onOpenWindow: function (xulWin) {
-      let domWin = xulWin.QueryInterface(Ci.nsIInterfaceRequestor)
-                   .getInterface(Ci.nsIDOMWindow);
-
-      whenWindowLoaded(domWin, function () {
-        let dialog = domWin.document.querySelector("dialog");
-        if (dialog) {
-          wm.removeListener(listener);
-          callback(dialog);
-        }
-      });
-    }
-  };
-
-  wm.addListener(listener);
-}
