@@ -8,9 +8,10 @@ const {Cc, Ci, Cu, Cr} = require("chrome");
 
 Cu.import("resource://gre/modules/Services.jsm");
 
-let promise = require("devtools/toolkit/deprecated-sync-thenables");
+let promise = require("resource://gre/modules/Promise.jsm").Promise;
 let EventEmitter = require("devtools/toolkit/event-emitter");
 let clipboard = require("sdk/clipboard");
+let {HostType} = require("devtools/framework/toolbox").Toolbox;
 
 loader.lazyGetter(this, "MarkupView", () => require("devtools/markupview/markup-view").MarkupView);
 loader.lazyGetter(this, "HTMLBreadcrumbs", () => require("devtools/inspector/breadcrumbs").HTMLBreadcrumbs);
@@ -48,8 +49,14 @@ const LAYOUT_CHANGE_TIMER = 250;
  *      Fired when a property is expanded in the computed rules view
  * - computed-view-property-collapsed
  *      Fired when a property is collapsed in the computed rules view
+ * - computed-view-sourcelinks-updated
+ *      Fired when the stylesheet source links have been updated (when switching
+ *      to source-mapped files)
  * - rule-view-refreshed
  *      Fired when the rule view updates to a new node
+ * - rule-view-sourcelinks-updated
+ *      Fired when the stylesheet source links have been updated (when switching
+ *      to source-mapped files)
  */
 function InspectorPanel(iframeWindow, toolbox) {
   this._toolbox = toolbox;
@@ -138,6 +145,9 @@ InspectorPanel.prototype = {
 
     this.breadcrumbs = new HTMLBreadcrumbs(this);
 
+    this.onToolboxHostChanged = this.onToolboxHostChanged.bind(this);
+    this._toolbox.on("host-changed", this.onToolboxHostChanged);
+
     if (this.target.isLocalTab) {
       this.browser = this.target.tab.linkedBrowser;
       this.scheduleLayoutChange = this.scheduleLayoutChange.bind(this);
@@ -249,10 +259,10 @@ InspectorPanel.prototype = {
       if (front) {
         return front;
       }
-      return this.walker.documentElement(this.walker.rootNode);
+      return this.walker.documentElement();
     }).then(node => {
-      if (walker !== this.walker) {
-        promise.reject(null);
+      if (hasNavigated()) {
+        return promise.reject("navigated; resolution of _defaultNode aborted");
       }
       this._defaultNode = node;
       return node;
@@ -273,13 +283,6 @@ InspectorPanel.prototype = {
     this._target = value;
   },
 
-  /**
-   * Expose gViewSourceUtils so that other tools can make use of them.
-   */
-  get viewSourceUtils() {
-    return this.panelWin.gViewSourceUtils;
-  },
-
   /**
    * Indicate that a tool has modified the state of the page.  Used to
    * decide whether to show the "are you sure you want to navigate"
@@ -307,7 +310,9 @@ InspectorPanel.prototype = {
    */
   setupSidebar: function InspectorPanel_setupSidebar() {
     let tabbox = this.panelDoc.querySelector("#inspector-sidebar");
-    this.sidebar = new ToolSidebar(tabbox, this, "inspector");
+    this.sidebar = new ToolSidebar(tabbox, this, "inspector", {
+      showAllTabsMenu: true
+    });
 
     let defaultTab = Services.prefs.getCharPref("devtools.inspector.activeSidebar");
 
@@ -335,9 +340,26 @@ InspectorPanel.prototype = {
                         "chrome://browser/content/devtools/layoutview/view.xhtml",
                         "layoutview" == defaultTab);
 
-    let ruleViewTab = this.sidebar.getTab("ruleview");
+    if (this.target.form.animationsActor) {
+      this.sidebar.addTab("animationinspector",
+                          "chrome://browser/content/devtools/animationinspector/animation-inspector.xhtml",
+                          "animationinspector" == defaultTab);
+    }
 
     this.sidebar.show();
+
+    this.setupSidebarToggle();
+  },
+
+  /**
+   * Add the expand/collapse behavior for the sidebar panel.
+   */
+  setupSidebarToggle: function() {
+    this._paneToggleButton = this.panelDoc.getElementById("inspector-pane-toggle");
+    this.onPaneToggleButtonClicked = this.onPaneToggleButtonClicked.bind(this);
+    this._paneToggleButton.addEventListener("mousedown",
+      this.onPaneToggleButtonClicked);
+    this.updatePaneToggleButton();
   },
 
   /**
@@ -380,6 +402,10 @@ InspectorPanel.prototype = {
    * reload
    */
   set selectionCssSelector(cssSelector = null) {
+    if (this._panelDestroyer) {
+      return;
+    }
+
     this._selectionCssSelector = {
       selector: cssSelector,
       url: this._target.url
@@ -541,21 +567,25 @@ InspectorPanel.prototype = {
     this.target.off("thread-paused", this.updateDebuggerPausedWarning);
     this.target.off("thread-resumed", this.updateDebuggerPausedWarning);
     this._toolbox.off("select", this.updateDebuggerPausedWarning);
+    this._toolbox.off("host-changed", this.onToolboxHostChanged);
 
     this.sidebar.off("select", this._setDefaultSidebar);
-    this.sidebar.destroy();
+    let sidebarDestroyer = this.sidebar.destroy();
     this.sidebar = null;
 
     this.nodemenu.removeEventListener("popupshowing", this._setupNodeMenu, true);
     this.nodemenu.removeEventListener("popuphiding", this._resetNodeMenu, true);
     this.breadcrumbs.destroy();
+    this._paneToggleButton.removeEventListener("mousedown",
+      this.onPaneToggleButtonClicked);
+    this._paneToggleButton = null;
     this.searchSuggestions.destroy();
     this.searchBox = null;
     this.selection.off("new-node-front", this.onNewSelection);
     this.selection.off("before-new-node", this.onBeforeNewSelection);
     this.selection.off("before-new-node-front", this.onBeforeNewSelection);
     this.selection.off("detached-front", this.onDetached);
-    this._panelDestroyer = this._destroyMarkup();
+    let markupDestroyer = this._destroyMarkup();
     this.panelWin.inspector = null;
     this.target = null;
     this.panelDoc = null;
@@ -566,6 +596,11 @@ InspectorPanel.prototype = {
     this.nodemenu = null;
     this._toolbox = null;
 
+    this._panelDestroyer = promise.all([
+      sidebarDestroyer,
+      markupDestroyer
+    ]);
+
     return this._panelDestroyer;
   },
 
@@ -631,24 +666,35 @@ InspectorPanel.prototype = {
       deleteNode.setAttribute("disabled", "true");
     }
 
-    // Disable / enable "Copy Unique Selector", "Copy inner HTML" &
-    // "Copy outer HTML" as appropriate
+    // Disable / enable "Copy Unique Selector", "Copy inner HTML",
+    // "Copy outer HTML" & "Scroll Into View" as appropriate
     let unique = this.panelDoc.getElementById("node-menu-copyuniqueselector");
     let copyInnerHTML = this.panelDoc.getElementById("node-menu-copyinner");
     let copyOuterHTML = this.panelDoc.getElementById("node-menu-copyouter");
+    let scrollIntoView = this.panelDoc.getElementById("node-menu-scrollnodeintoview");
+
+    this._target.actorHasMethod("domnode", "scrollIntoView").then(value => {
+      scrollIntoView.hidden = !value;
+    });
+
     if (isSelectionElement) {
       unique.removeAttribute("disabled");
       copyInnerHTML.removeAttribute("disabled");
       copyOuterHTML.removeAttribute("disabled");
+      scrollIntoView.removeAttribute("disabled");
     } else {
       unique.setAttribute("disabled", "true");
       copyInnerHTML.setAttribute("disabled", "true");
       copyOuterHTML.setAttribute("disabled", "true");
+      scrollIntoView.setAttribute("disabled", "true");
     }
     if (!this.canGetUniqueSelector) {
       unique.hidden = true;
     }
 
+    // Enable/Disable the link open/copy items.
+    this._setupNodeLinkMenu();
+
     // Enable the "edit HTML" item if the selection is an element and the root
     // actor has the appropriate trait (isOuterHTMLEditable)
     let editHTML = this.panelDoc.getElementById("node-menu-edithtml");
@@ -707,6 +753,63 @@ InspectorPanel.prototype = {
     }
   },
 
+  /**
+   * Link menu items can be shown or hidden depending on the context and
+   * selected node, and their labels can vary.
+   */
+  _setupNodeLinkMenu: function InspectorPanel_setupNodeLinkMenu() {
+    let linkSeparator = this.panelDoc.getElementById("node-menu-link-separator");
+    let linkFollow = this.panelDoc.getElementById("node-menu-link-follow");
+    let linkCopy = this.panelDoc.getElementById("node-menu-link-copy");
+
+    // Hide all by default.
+    linkSeparator.setAttribute("hidden", "true");
+    linkFollow.setAttribute("hidden", "true");
+    linkCopy.setAttribute("hidden", "true");
+
+    // Get information about the right-clicked node.
+    let popupNode = this.panelDoc.popupNode;
+    if (!popupNode || !popupNode.classList.contains("link")) {
+      return;
+    }
+
+    let type = popupNode.dataset.type;
+    if (type === "uri" || type === "cssresource" || type === "jsresource") {
+      // First make sure the target can resolve relative URLs.
+      this.target.actorHasMethod("inspector", "resolveRelativeURL").then(canResolve => {
+        if (!canResolve) {
+          return;
+        }
+
+        linkSeparator.removeAttribute("hidden");
+
+        // Links can't be opened in new tabs in the browser toolbox.
+        if (type === "uri" && !this.target.chrome) {
+          linkFollow.removeAttribute("hidden");
+          linkFollow.setAttribute("label", this.strings.GetStringFromName(
+            "inspector.menu.openUrlInNewTab.label"));
+        } else if (type === "cssresource") {
+          linkFollow.removeAttribute("hidden");
+          linkFollow.setAttribute("label", this.toolboxStrings.GetStringFromName(
+            "toolbox.viewCssSourceInStyleEditor.label"));
+        } else if (type === "jsresource") {
+          linkFollow.removeAttribute("hidden");
+          linkFollow.setAttribute("label", this.toolboxStrings.GetStringFromName(
+            "toolbox.viewJsSourceInDebugger.label"));
+        }
+
+        linkCopy.removeAttribute("hidden");
+        linkCopy.setAttribute("label", this.strings.GetStringFromName(
+          "inspector.menu.copyUrlToClipboard.label"));
+      }, console.error);
+    } else if (type === "idref") {
+      linkSeparator.removeAttribute("hidden");
+      linkFollow.removeAttribute("hidden");
+      linkFollow.setAttribute("label", this.strings.formatStringFromName(
+        "inspector.menu.selectElement.label", [popupNode.dataset.link], 1));
+    }
+  },
+
   _initMarkup: function InspectorPanel_initMarkup() {
     let doc = this.panelDoc;
 
@@ -767,6 +870,53 @@ InspectorPanel.prototype = {
     return destroyPromise;
   },
 
+  /**
+   * When the type of toolbox host changes.
+   */
+  onToolboxHostChanged: function() {
+    this.updatePaneToggleButton();
+  },
+
+  /**
+   * When the pane toggle button is clicked, toggle the pane, change the button
+   * state and tooltip.
+   */
+  onPaneToggleButtonClicked: function(e) {
+    let sidePane = this.panelDoc.querySelector("#inspector-sidebar");
+    let button = this._paneToggleButton;
+    let isVisible = !button.hasAttribute("pane-collapsed");
+
+    // Make sure the sidebar has a width attribute before collapsing because
+    // ViewHelpers needs it.
+    if (isVisible && !sidePane.hasAttribute("width")) {
+      sidePane.setAttribute("width", sidePane.getBoundingClientRect().width);
+    }
+
+    ViewHelpers.togglePane({
+      visible: !isVisible,
+      animated: true,
+      delayed: true
+    }, sidePane);
+
+    if (isVisible) {
+      button.setAttribute("pane-collapsed", "");
+      button.setAttribute("tooltiptext",
+        this.strings.GetStringFromName("inspector.expandPane"));
+    } else {
+      button.removeAttribute("pane-collapsed");
+      button.setAttribute("tooltiptext",
+        this.strings.GetStringFromName("inspector.collapsePane"));
+    }
+  },
+
+  /**
+   * Update the pane toggle button visibility depending on the toolbox host type.
+   */
+  updatePaneToggleButton: function() {
+    this._paneToggleButton.setAttribute("hidden",
+      this._toolbox.hostType === HostType.SIDE);
+  },
+
   /**
    * Toggle a pseudo class.
    */
@@ -791,7 +941,7 @@ InspectorPanel.prototype = {
       let jsterm = panel.hud.jsterm;
 
       jsterm.execute("inspect($0)");
-      jsterm.focusInput();
+      jsterm.inputNode.focus();
     });
   },
 
@@ -916,6 +1066,18 @@ InspectorPanel.prototype = {
     }).then(null, console.error);
   },
 
+  /**
+   * Scroll the node into view.
+   */
+  scrollNodeIntoView: function InspectorPanel_scrollNodeIntoView()
+  {
+    if (!this.selection.isNode()) {
+      return;
+    }
+
+    this.selection.nodeFront.scrollIntoView();
+  },
+
   /**
    * Delete the selected node.
    */
@@ -935,6 +1097,57 @@ InspectorPanel.prototype = {
     }
   },
 
+  /**
+   * This method is here for the benefit of the node-menu-link-follow menu item
+   * in the inspector contextual-menu. It's behavior depends on which node was
+   * right-clicked when the menu was opened.
+   */
+  followAttributeLink: function InspectorPanel_followLink(e) {
+    let type = this.panelDoc.popupNode.dataset.type;
+    let link = this.panelDoc.popupNode.dataset.link;
+
+    if (type === "uri" || type === "cssresource" || type === "jsresource") {
+      // Open link in a new tab.
+      // When the inspector menu was setup on click (see _setupNodeLinkMenu), we
+      // already checked that resolveRelativeURL existed.
+      this.inspector.resolveRelativeURL(link, this.selection.nodeFront).then(url => {
+        if (type === "uri") {
+          let browserWin = this.target.tab.ownerDocument.defaultView;
+          browserWin.openUILinkIn(url, "tab");
+        } else if (type === "cssresource") {
+          return this.toolbox.viewSourceInStyleEditor(url);
+        } else if (type === "jsresource") {
+          return this.toolbox.viewSourceInDebugger(url);
+        }
+      }).catch(e => console.error(e));
+    } else if (type == "idref") {
+      // Select the node in the same document.
+      this.walker.document(this.selection.nodeFront).then(doc => {
+        return this.walker.querySelector(doc, "#" + CSS.escape(link)).then(node => {
+          if (!node) {
+            this.emit("idref-attribute-link-failed");
+            return;
+          }
+          this.selection.setNodeFront(node);
+        });
+      }).catch(e => console.error(e));
+    }
+  },
+
+  /**
+   * This method is here for the benefit of the node-menu-link-copy menu item
+   * in the inspector contextual-menu. It's behavior depends on which node was
+   * right-clicked when the menu was opened.
+   */
+  copyAttributeLink: function InspectorPanel_copyLink(e) {
+    let link = this.panelDoc.popupNode.dataset.link;
+    // When the inspector menu was setup on click (see _setupNodeLinkMenu), we
+    // already checked that resolveRelativeURL existed.
+    this.inspector.resolveRelativeURL(link, this.selection.nodeFront).then(url => {
+      clipboardHelper.copyString(url);
+    }, console.error);
+  },
+
   /**
   * Trigger a high-priority layout change for things that need to be
   * updated immediately
@@ -978,18 +1191,21 @@ InspectorPanel.prototype = {
 /////////////////////////////////////////////////////////////////////////
 //// Initializers
 
-loader.lazyGetter(InspectorPanel.prototype, "strings",
-  function () {
-    return Services.strings.createBundle(
-            "chrome://browser/locale/devtools/inspector.properties");
-  });
+loader.lazyGetter(InspectorPanel.prototype, "strings", function () {
+  return Services.strings.createBundle(
+    "chrome://browser/locale/devtools/inspector.properties");
+});
+
+loader.lazyGetter(InspectorPanel.prototype, "toolboxStrings", function () {
+  return Services.strings.createBundle(
+    "chrome://browser/locale/devtools/toolbox.properties");
+});
 
 loader.lazyGetter(this, "clipboardHelper", function() {
   return Cc["@mozilla.org/widget/clipboardhelper;1"].
     getService(Ci.nsIClipboardHelper);
 });
 
-
 loader.lazyGetter(this, "DOMUtils", function () {
   return Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);
 });
