@@ -114,12 +114,26 @@ let testcases = [ {
     input: "192.168.10.110",
     fixedURI: "http://192.168.10.110/",
     protocolChange: true,
+  }, {
+    input: "192.168.10.110/123",
+    fixedURI: "http://192.168.10.110/123",
+    protocolChange: true,
+  }, {
+    input: "192.168.10.110/123foo",
+    fixedURI: "http://192.168.10.110/123foo",
+    protocolChange: true,
+  }, {
+    input: "192.168.10.110:1234/123",
+    fixedURI: "http://192.168.10.110:1234/123",
+    protocolChange: true,
+  }, {
+    input: "192.168.10.110:1234/123foo",
+    fixedURI: "http://192.168.10.110:1234/123foo",
+    protocolChange: true,
   }, {
     input: "1.2.3",
     fixedURI: "http://1.2.3/",
-    keywordLookup: true,
     protocolChange: true,
-    affectedByDNSForSingleHosts: true,
   }, {
     input: "1.2.3/",
     fixedURI: "http://1.2.3/",
@@ -128,12 +142,14 @@ let testcases = [ {
     input: "1.2.3/foo",
     fixedURI: "http://1.2.3/foo",
     protocolChange: true,
+  }, {
+    input: "1.2.3/123",
+    fixedURI: "http://1.2.3/123",
+    protocolChange: true,
   }, {
     input: "1.2.3:8000",
     fixedURI: "http://1.2.3:8000/",
-    keywordLookup: true,
     protocolChange: true,
-    affectedByDNSForSingleHosts: true,
   }, {
     input: "1.2.3:8000/",
     fixedURI: "http://1.2.3:8000/",
@@ -142,6 +158,10 @@ let testcases = [ {
     input: "1.2.3:8000/foo",
     fixedURI: "http://1.2.3:8000/foo",
     protocolChange: true,
+  }, {
+    input: "1.2.3:8000/123",
+    fixedURI: "http://1.2.3:8000/123",
+    protocolChange: true,
   }, {
     input: "http://1.2.3",
     fixedURI: "http://1.2.3/",
@@ -375,6 +395,14 @@ let testcases = [ {
     protocolChange: true,
     affectedByWhitelist: true,
     affectedByDNSForSingleHosts: true,
+  }, {
+    input: "5/2",
+    fixedURI: "http://5/2",
+    alternateURI: "http://www.5.com/2",
+    keywordLookup: true,
+    protocolChange: true,
+    affectedByWhitelist: true,
+    affectedByDNSForSingleHosts: true,
   }, {
     input: "moz ?.::%27",
     keywordLookup: true,
@@ -448,6 +476,32 @@ let testcases = [ {
     keywordLookup: true,
     protocolChange: true,
     affectedByWhitelist: true
+  }, {
+    input: "mozilla/",
+    fixedURI: "http://mozilla/",
+    alternateURI: "http://www.mozilla.com/",
+    protocolChange: true,
+    affectedByWhitelist: true,
+  }, {
+    input: "mozilla",
+    fixedURI: "http://mozilla/",
+    alternateURI: "http://www.mozilla.com/",
+    protocolChange: true,
+    keywordLookup: true,
+    affectedByWhitelist: true,
+    affectedByDNSForSingleHosts: true,
+  }, {
+    input: "mozilla5/2",
+    fixedURI: "http://mozilla5/2",
+    alternateURI: "http://www.mozilla5.com/2",
+    protocolChange: true,
+    affectedByWhitelist: true,
+  }, {
+    input: "mozilla/foo",
+    fixedURI: "http://mozilla/foo",
+    alternateURI: "http://www.mozilla.com/foo",
+    protocolChange: true,
+    affectedByWhitelist: true,
   }];
 
 if (Services.appinfo.OS.toLowerCase().startsWith("win")) {
@@ -467,10 +521,8 @@ if (Services.appinfo.OS.toLowerCase().startsWith("win")) {
     input: "mozilla\\",
     fixedURI: "http://mozilla/",
     alternateURI: "http://www.mozilla.com/",
-    keywordLookup: true,
     protocolChange: true,
     affectedByWhitelist: true,
-    affectedByDNSForSingleHosts: true,
   });
 } else {
   testcases.push({
@@ -589,14 +641,14 @@ function do_single_test_run() {
       if (couldDoKeywordLookup) {
         if (expectKeywordLookup) {
           if (!affectedByWhitelist || (affectedByWhitelist && !inWhitelist)) {
-            let urlparamInput = encodeURIComponent(sanitize(testInput)).replace("%20", "+", "g");
+            let urlparamInput = encodeURIComponent(sanitize(testInput)).replace(/%20/g, "+");
             // If the input starts with `?`, then info.preferredURI.spec will omit it
             // In order to test this behaviour, remove `?` only if it is the first character
             if (urlparamInput.startsWith("%3F")) {
               urlparamInput = urlparamInput.replace("%3F", "");
             }
             let searchURL = kSearchEngineURL.replace("{searchTerms}", urlparamInput);
-            let spec = info.preferredURI.spec.replace("%27", "'", "g");
+            let spec = info.preferredURI.spec.replace(/%27/g, "'");
             do_check_eq(spec, searchURL);
           } else {
             do_check_eq(info.preferredURI, null);
