@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -16,13 +17,17 @@
 #include "nsRuleNode.h"
 #include "nsRuleData.h"
 
+// For IsPictureEnabled() -- the candidate parser needs to be aware of sizes
+// support being enabled
+#include "HTMLPictureElement.h"
+
 using namespace mozilla;
 using namespace mozilla::dom;
 
 namespace mozilla {
 namespace dom {
 
-NS_IMPL_CYCLE_COLLECTION(ResponsiveImageSelector, mContent)
+NS_IMPL_CYCLE_COLLECTION(ResponsiveImageSelector, mOwnerNode)
 
 NS_IMPL_CYCLE_COLLECTION_ROOT_NATIVE(ResponsiveImageSelector, AddRef)
 NS_IMPL_CYCLE_COLLECTION_UNROOT_NATIVE(ResponsiveImageSelector, Release)
@@ -35,12 +40,19 @@ ParseInteger(const nsAString& aString, int32_t& aInt)
   return !(parseResult &
            ( nsContentUtils::eParseHTMLInteger_Error |
              nsContentUtils::eParseHTMLInteger_DidNotConsumeAllInput |
-             nsContentUtils::eParseHTMLInteger_IsPercent ));
+             nsContentUtils::eParseHTMLInteger_IsPercent |
+             nsContentUtils::eParseHTMLInteger_NonStandard ));
 }
 
 ResponsiveImageSelector::ResponsiveImageSelector(nsIContent *aContent)
-  : mContent(aContent),
-    mBestCandidateIndex(-1)
+  : mOwnerNode(aContent),
+    mSelectedCandidateIndex(-1)
+{
+}
+
+ResponsiveImageSelector::ResponsiveImageSelector(nsIDocument *aDocument)
+  : mOwnerNode(aDocument),
+    mSelectedCandidateIndex(-1)
 {
 }
 
@@ -51,21 +63,22 @@ ResponsiveImageSelector::~ResponsiveImageSelector()
 bool
 ResponsiveImageSelector::SetCandidatesFromSourceSet(const nsAString & aSrcSet)
 {
-  nsIDocument* doc = mContent ? mContent->OwnerDoc() : nullptr;
-  nsCOMPtr<nsIURI> docBaseURI = mContent ? mContent->GetBaseURI() : nullptr;
+  ClearSelectedCandidate();
 
-  if (!mContent || !doc || !docBaseURI) {
+  nsCOMPtr<nsIURI> docBaseURI = mOwnerNode ? mOwnerNode->GetBaseURI() : nullptr;
+
+  if (!docBaseURI) {
     MOZ_ASSERT(false,
-               "Should not be parsing SourceSet without a content and document");
+               "Should not be parsing SourceSet without a document");
     return false;
   }
 
   // Preserve the default source if we have one, it has a separate setter.
   uint32_t prevNumCandidates = mCandidates.Length();
-  nsCOMPtr<nsIURI> defaultURL;
+  nsString defaultURLString;
   if (prevNumCandidates && (mCandidates[prevNumCandidates - 1].Type() ==
                             ResponsiveImageCandidate::eCandidateType_Default)) {
-    defaultURL = mCandidates[prevNumCandidates - 1].URL();
+    defaultURLString = mCandidates[prevNumCandidates - 1].URLString();
   }
 
   mCandidates.Clear();
@@ -76,10 +89,12 @@ ResponsiveImageSelector::SetCandidatesFromSourceSet(const nsAString & aSrcSet)
 
   // Read URL / descriptor pairs
   while (iter != end) {
-    nsAString::const_iterator url, desc;
+    nsAString::const_iterator url, urlEnd, descriptor;
 
-    // Skip whitespace
-    for (; iter != end && nsContentUtils::IsHTMLWhitespace(*iter); ++iter);
+    // Skip whitespace and commas.
+    // Extra commas at this point are a non-fatal syntax error.
+    for (; iter != end && (nsContentUtils::IsHTMLWhitespace(*iter) ||
+                           *iter == char16_t(',')); ++iter);
 
     if (iter == end) {
       break;
@@ -90,70 +105,36 @@ ResponsiveImageSelector::SetCandidatesFromSourceSet(const nsAString & aSrcSet)
     // Find end of url
     for (;iter != end && !nsContentUtils::IsHTMLWhitespace(*iter); ++iter);
 
-    desc = iter;
-
-    // Find end of descriptor
-    for (; iter != end && *iter != char16_t(','); ++iter);
-    const nsDependentSubstring &descriptor = Substring(desc, iter);
-
-    nsresult rv;
-    nsCOMPtr<nsIURI> candidateURL;
-    rv = nsContentUtils::NewURIWithDocumentCharset(getter_AddRefs(candidateURL),
-                                                   Substring(url, desc),
-                                                   doc,
-                                                   docBaseURI);
-    if (NS_SUCCEEDED(rv) && candidateURL) {
-      NS_TryToSetImmutable(candidateURL);
-      ResponsiveImageCandidate candidate;
-      if (candidate.SetParamaterFromDescriptor(descriptor)) {
-        candidate.SetURL(candidateURL);
-        AppendCandidateIfUnique(candidate);
+    // Omit trailing commas from URL.
+    // Multiple commas are a non-fatal error.
+    while (iter != url) {
+      if (*(--iter) != char16_t(',')) {
+        iter++;
+        break;
       }
     }
 
-    // Advance past comma
-    if (iter != end) {
-      ++iter;
+    const nsDependentSubstring &urlStr = Substring(url, iter);
+
+    MOZ_ASSERT(url != iter, "Shouldn't have empty URL at this point");
+
+    ResponsiveImageCandidate candidate;
+    if (candidate.ConsumeDescriptors(iter, end)) {
+      candidate.SetURLSpec(urlStr);
+      AppendCandidateIfUnique(candidate);
     }
   }
 
   bool parsedCandidates = mCandidates.Length() > 0;
 
   // Re-add default to end of list
-  if (defaultURL) {
-    AppendDefaultCandidate(defaultURL);
+  if (!defaultURLString.IsEmpty()) {
+    AppendDefaultCandidate(defaultURLString);
   }
 
   return parsedCandidates;
 }
 
-nsresult
-ResponsiveImageSelector::SetDefaultSource(const nsAString & aSpec)
-{
-  nsIDocument* doc = mContent ? mContent->OwnerDoc() : nullptr;
-  nsCOMPtr<nsIURI> docBaseURI = mContent ? mContent->GetBaseURI() : nullptr;
-
-  if (!mContent || !doc || !docBaseURI) {
-    MOZ_ASSERT(false,
-               "Should not be calling this without a content and document");
-    return NS_ERROR_UNEXPECTED;
-  }
-
-  if (aSpec.IsEmpty()) {
-    SetDefaultSource(nullptr);
-    return NS_OK;
-  }
-
-  nsresult rv;
-  nsCOMPtr<nsIURI> candidateURL;
-  rv = nsContentUtils::NewURIWithDocumentCharset(getter_AddRefs(candidateURL),
-                                                 aSpec, doc, docBaseURI);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  SetDefaultSource(candidateURL);
-  return NS_OK;
-}
-
 uint32_t
 ResponsiveImageSelector::NumCandidates(bool aIncludeDefault)
 {
@@ -169,31 +150,49 @@ ResponsiveImageSelector::NumCandidates(bool aIncludeDefault)
   return candidates;
 }
 
-void
-ResponsiveImageSelector::SetDefaultSource(nsIURI *aURL)
+nsIContent*
+ResponsiveImageSelector::Content()
 {
+  return mOwnerNode->IsContent() ? mOwnerNode->AsContent() : nullptr;
+}
+
+nsIDocument*
+ResponsiveImageSelector::Document()
+{
+  return mOwnerNode->OwnerDoc();
+}
+
+void
+ResponsiveImageSelector::SetDefaultSource(const nsAString& aURLString)
+{
+  ClearSelectedCandidate();
+
   // Check if the last element of our candidates is a default
   int32_t candidates = mCandidates.Length();
   if (candidates && (mCandidates[candidates - 1].Type() ==
                      ResponsiveImageCandidate::eCandidateType_Default)) {
     mCandidates.RemoveElementAt(candidates - 1);
-    if (mBestCandidateIndex == candidates - 1) {
-      mBestCandidateIndex = -1;
-    }
   }
 
   // Add new default if set
-  if (aURL) {
-    AppendDefaultCandidate(aURL);
+  if (!aURLString.IsEmpty()) {
+    AppendDefaultCandidate(aURLString);
   }
 }
 
+void
+ResponsiveImageSelector::ClearSelectedCandidate()
+{
+  mSelectedCandidateIndex = -1;
+  mSelectedCandidateURL = nullptr;
+}
+
 bool
 ResponsiveImageSelector::SetSizesFromDescriptor(const nsAString & aSizes)
 {
+  ClearSelectedCandidate();
   mSizeQueries.Clear();
   mSizeValues.Clear();
-  mBestCandidateIndex = -1;
 
   nsCSSParser cssParser;
 
@@ -224,41 +223,48 @@ ResponsiveImageSelector::AppendCandidateIfUnique(const ResponsiveImageCandidate
     }
   }
 
-  mBestCandidateIndex = -1;
   mCandidates.AppendElement(aCandidate);
 }
 
 void
-ResponsiveImageSelector::AppendDefaultCandidate(nsIURI *aURL)
+ResponsiveImageSelector::AppendDefaultCandidate(const nsAString& aURLString)
 {
-  NS_ENSURE_TRUE(aURL, /* void */);
+  NS_ENSURE_TRUE(!aURLString.IsEmpty(), /* void */);
 
   ResponsiveImageCandidate defaultCandidate;
   defaultCandidate.SetParameterDefault();
-  defaultCandidate.SetURL(aURL);
+  defaultCandidate.SetURLSpec(aURLString);
   // We don't use MaybeAppend since we want to keep this even if it can never
   // match, as it may if the source set changes.
-  mBestCandidateIndex = -1;
   mCandidates.AppendElement(defaultCandidate);
 }
 
 already_AddRefed<nsIURI>
 ResponsiveImageSelector::GetSelectedImageURL()
 {
-  int bestIndex = GetBestCandidateIndex();
-  if (bestIndex < 0) {
-    return nullptr;
+  SelectImage();
+
+  nsCOMPtr<nsIURI> url = mSelectedCandidateURL;
+  return url.forget();
+}
+
+bool
+ResponsiveImageSelector::GetSelectedImageURLSpec(nsAString& aResult)
+{
+  SelectImage();
+
+  if (mSelectedCandidateIndex == -1) {
+    return false;
   }
 
-  nsCOMPtr<nsIURI> bestURL = mCandidates[bestIndex].URL();
-  MOZ_ASSERT(bestURL, "Shouldn't have candidates with no URL in the array");
-  return bestURL.forget();
+  aResult.Assign(mCandidates[mSelectedCandidateIndex].URLString());
+  return true;
 }
 
 double
 ResponsiveImageSelector::GetSelectedImageDensity()
 {
-  int bestIndex = GetBestCandidateIndex();
+  int bestIndex = GetSelectedCandidateIndex();
   if (bestIndex < 0) {
     return 1.0;
   }
@@ -269,35 +275,27 @@ ResponsiveImageSelector::GetSelectedImageDensity()
 bool
 ResponsiveImageSelector::SelectImage(bool aReselect)
 {
-  if (!aReselect && mBestCandidateIndex != -1) {
+  if (!aReselect && mSelectedCandidateIndex != -1) {
     // Already have selection
     return false;
   }
 
-  int oldBest = mBestCandidateIndex;
-  mBestCandidateIndex = -1;
-  return GetBestCandidateIndex() != oldBest;
-}
-
-int
-ResponsiveImageSelector::GetBestCandidateIndex()
-{
-  if (mBestCandidateIndex != -1) {
-    return mBestCandidateIndex;
-  }
+  int oldBest = mSelectedCandidateIndex;
+  ClearSelectedCandidate();
 
   int numCandidates = mCandidates.Length();
   if (!numCandidates) {
-    return -1;
+    return oldBest != -1;
   }
 
-  nsIDocument* doc = mContent ? mContent->OwnerDoc() : nullptr;
+  nsIDocument* doc = Document();
   nsIPresShell *shell = doc ? doc->GetShell() : nullptr;
   nsPresContext *pctx = shell ? shell->GetPresContext() : nullptr;
+  nsCOMPtr<nsIURI> baseURI = mOwnerNode ? mOwnerNode->GetBaseURI() : nullptr;
 
-  if (!pctx) {
-    MOZ_ASSERT(false, "Unable to find document prescontext");
-    return -1;
+  if (!pctx || !doc || !baseURI) {
+    MOZ_ASSERT(false, "Unable to find document prescontext and base URI");
+    return oldBest != -1;
   }
 
   double displayDensity = pctx->CSSPixelsToDevPixels(1.0f);
@@ -344,15 +342,33 @@ ResponsiveImageSelector::GetBestCandidateIndex()
   }
 
   MOZ_ASSERT(bestIndex >= 0 && bestIndex < numCandidates);
-  mBestCandidateIndex = bestIndex;
-  return bestIndex;
+
+  // Resolve URL
+  nsresult rv;
+  const nsAString& urlStr = mCandidates[bestIndex].URLString();
+  nsCOMPtr<nsIURI> candidateURL;
+  rv = nsContentUtils::NewURIWithDocumentCharset(getter_AddRefs(candidateURL),
+                                                 urlStr, doc, baseURI);
+
+  mSelectedCandidateURL = NS_SUCCEEDED(rv) ? candidateURL : nullptr;
+  mSelectedCandidateIndex = bestIndex;
+
+  return mSelectedCandidateIndex != oldBest;
+}
+
+int
+ResponsiveImageSelector::GetSelectedCandidateIndex()
+{
+  SelectImage();
+
+  return mSelectedCandidateIndex;
 }
 
 bool
 ResponsiveImageSelector::ComputeFinalWidthForCurrentViewport(int32_t *aWidth)
 {
   unsigned int numSizes = mSizeQueries.Length();
-  nsIDocument* doc = mContent ? mContent->OwnerDoc() : nullptr;
+  nsIDocument* doc = Document();
   nsIPresShell *presShell = doc ? doc->GetShell() : nullptr;
   nsPresContext *pctx = presShell ? presShell->GetPresContext() : nullptr;
 
@@ -393,9 +409,9 @@ ResponsiveImageCandidate::ResponsiveImageCandidate()
   mValue.mDensity = 1.0;
 }
 
-ResponsiveImageCandidate::ResponsiveImageCandidate(nsIURI *aURL,
+ResponsiveImageCandidate::ResponsiveImageCandidate(const nsAString& aURLString,
                                                    double aDensity)
-  : mURL(aURL)
+  : mURLString(aURLString)
 {
   mType = eCandidateType_Density;
   mValue.mDensity = aDensity;
@@ -403,9 +419,9 @@ ResponsiveImageCandidate::ResponsiveImageCandidate(nsIURI *aURL,
 
 
 void
-ResponsiveImageCandidate::SetURL(nsIURI *aURL)
+ResponsiveImageCandidate::SetURLSpec(const nsAString& aURLString)
 {
-  mURL = aURL;
+  mURLString = aURLString;
 }
 
 void
@@ -426,6 +442,15 @@ ResponsiveImageCandidate::SetParameterDefault()
   mValue.mDensity = 1.0;
 }
 
+void
+ResponsiveImageCandidate::SetParameterInvalid()
+{
+  mType = eCandidateType_Invalid;
+  // mValue shouldn't actually be used for this type, but set it to default
+  // anyway
+  mValue.mDensity = 1.0;
+}
+
 void
 ResponsiveImageCandidate::SetParameterAsDensity(double aDensity)
 {
@@ -435,76 +460,184 @@ ResponsiveImageCandidate::SetParameterAsDensity(double aDensity)
   mValue.mDensity = aDensity;
 }
 
-bool
-ResponsiveImageCandidate::SetParamaterFromDescriptor(const nsAString & aDescriptor)
+// Represents all supported descriptors for a ResponsiveImageCandidate, though
+// there is no candidate type that uses all of these. This should generally
+// match the mValue union of ResponsiveImageCandidate.
+struct ResponsiveImageDescriptors {
+  ResponsiveImageDescriptors()
+    : mInvalid(false) {};
+
+  Maybe<double> mDensity;
+  Maybe<int32_t> mWidth;
+  // We don't support "h" descriptors yet and they are not spec'd, but the
+  // current spec does specify that they can be silently ignored (whereas
+  // entirely unknown descriptors cause us to invalidate the candidate)
+  Maybe<int32_t> mFutureCompatHeight;
+  // If this descriptor set is bogus, e.g. a value was added twice (and thus
+  // dropped) or an unknown descriptor was added.
+  bool mInvalid;
+
+  void AddDescriptor(const nsAString& aDescriptor);
+  bool Valid();
+  // Use the current set of descriptors to configure a candidate
+  void FillCandidate(ResponsiveImageCandidate &aCandidate);
+};
+
+// Try to parse a single descriptor from a string. If value already set or
+// unknown, sets invalid flag.
+// This corresponds to the descriptor "Descriptor parser" step in:
+// https://html.spec.whatwg.org/#parse-a-srcset-attribute
+void
+ResponsiveImageDescriptors::AddDescriptor(const nsAString& aDescriptor)
 {
-  // Valid input values must be positive, using -1 for not-set
-  double density = -1.0;
-  int32_t width = -1;
+  if (aDescriptor.IsEmpty()) {
+    return;
+  }
 
-  nsAString::const_iterator iter, end;
-  aDescriptor.BeginReading(iter);
-  aDescriptor.EndReading(end);
-
-  // Parse descriptor list
-  // We currently only support a single density descriptor of the form:
-  //   <floating-point number>x
-  // Silently ignore other descriptors in the list for forward-compat
-  while (iter != end) {
-    // Skip initial whitespace
-    for (; iter != end && nsContentUtils::IsHTMLWhitespace(*iter); ++iter);
-    if (iter == end) {
-      break;
-    }
-
-    // Find end of type
-    nsAString::const_iterator start = iter;
-    for (; iter != end && !nsContentUtils::IsHTMLWhitespace(*iter); ++iter);
-
-    if (start == iter) {
-      // Empty descriptor
-      break;
-    }
-
-    // Iter is at end of descriptor, type is single character previous to that.
-    // Safe because we just verified that iter > start
-    --iter;
-    nsAString::const_iterator type(iter);
-    ++iter;
-
-    const nsDependentSubstring& valStr = Substring(start, type);
-    if (*type == char16_t('w')) {
-      int32_t possibleWidth;
-      if (width == -1 && density == -1.0) {
-        if (ParseInteger(valStr, possibleWidth) && possibleWidth > 0) {
-          width = possibleWidth;
-        }
+  // All currently supported descriptors end with an identifying character.
+  nsAString::const_iterator descStart, descType;
+  aDescriptor.BeginReading(descStart);
+  aDescriptor.EndReading(descType);
+  descType--;
+  const nsDependentSubstring& valueStr = Substring(descStart, descType);
+  if (*descType == char16_t('w')) {
+    int32_t possibleWidth;
+    // If the value is not a valid non-negative integer, it doesn't match this
+    // descriptor, fall through.
+    if (ParseInteger(valueStr, possibleWidth) && possibleWidth >= 0) {
+      if (possibleWidth != 0 && HTMLPictureElement::IsPictureEnabled() &&
+          mWidth.isNothing() && mDensity.isNothing()) {
+        mWidth.emplace(possibleWidth);
       } else {
-        return false;
+        // Valid width descriptor, but width or density were already seen, sizes
+        // support isn't enabled, or it parsed to 0, which is an error per spec
+        mInvalid = true;
       }
-    } else if (*type == char16_t('x')) {
-      if (width == -1 && density == -1.0) {
-        nsresult rv;
-        double possibleDensity = PromiseFlatString(valStr).ToDouble(&rv);
-        if (NS_SUCCEEDED(rv) && possibleDensity > 0.0) {
-          density = possibleDensity;
-        }
+
+      return;
+    }
+  } else if (*descType == char16_t('h')) {
+    int32_t possibleHeight;
+    // If the value is not a valid non-negative integer, it doesn't match this
+    // descriptor, fall through.
+    if (ParseInteger(valueStr, possibleHeight) && possibleHeight >= 0) {
+      if (possibleHeight != 0 && mFutureCompatHeight.isNothing() &&
+          mDensity.isNothing()) {
+        mFutureCompatHeight.emplace(possibleHeight);
       } else {
-        return false;
+        // Valid height descriptor, but height or density were already seen, or
+        // it parsed to zero, which is an error per spec
+        mInvalid = true;
       }
+
+      return;
+    }
+  } else if (*descType == char16_t('x')) {
+    // If the value is not a valid floating point number, it doesn't match this
+    // descriptor, fall through.
+    nsresult rv;
+    double possibleDensity = PromiseFlatString(valueStr).ToDouble(&rv);
+    if (NS_SUCCEEDED(rv)) {
+      if (possibleDensity >= 0.0 &&
+          mWidth.isNothing() &&
+          mDensity.isNothing() &&
+          mFutureCompatHeight.isNothing()) {
+        mDensity.emplace(possibleDensity);
+      } else {
+        // Valid density descriptor, but height or width or density were already
+        // seen, or it parsed to less than zero, which is an error per spec
+        mInvalid = true;
+      }
+
+      return;
     }
   }
 
-  if (width != -1) {
-    SetParameterAsComputedWidth(width);
-  } else if (density != -1.0) {
-    SetParameterAsDensity(density);
+  // Matched no known descriptor, mark this descriptor set invalid
+  mInvalid = true;
+}
+
+bool
+ResponsiveImageDescriptors::Valid()
+{
+  return !mInvalid && !(mFutureCompatHeight.isSome() && mWidth.isNothing());
+}
+
+void
+ResponsiveImageDescriptors::FillCandidate(ResponsiveImageCandidate &aCandidate)
+{
+  if (!Valid()) {
+    aCandidate.SetParameterInvalid();
+  } else if (mWidth.isSome()) {
+    MOZ_ASSERT(mDensity.isNothing()); // Shouldn't be valid
+
+    aCandidate.SetParameterAsComputedWidth(*mWidth);
+  } else if (mDensity.isSome()) {
+    MOZ_ASSERT(mWidth.isNothing()); // Shouldn't be valid
+
+    aCandidate.SetParameterAsDensity(*mDensity);
   } else {
-    // No valid descriptors -> 1.0 density
-    SetParameterAsDensity(1.0);
+    // A valid set of descriptors with no density nor width (e.g. an empty set)
+    // becomes 1.0 density, per spec
+    aCandidate.SetParameterAsDensity(1.0);
+  }
+}
+
+bool
+ResponsiveImageCandidate::ConsumeDescriptors(nsAString::const_iterator& aIter,
+                                             const nsAString::const_iterator& aIterEnd)
+{
+  nsAString::const_iterator &iter = aIter;
+  const nsAString::const_iterator &end  = aIterEnd;
+
+  bool inParens = false;
+
+  ResponsiveImageDescriptors descriptors;
+
+  // Parse descriptor list.
+  // This corresponds to the descriptor parsing loop from:
+  // https://html.spec.whatwg.org/#parse-a-srcset-attribute
+
+  // Skip initial whitespace
+  for (; iter != end && nsContentUtils::IsHTMLWhitespace(*iter); ++iter);
+
+  nsAString::const_iterator currentDescriptor = iter;
+
+  for (;; iter++) {
+    if (iter == end) {
+      descriptors.AddDescriptor(Substring(currentDescriptor, iter));
+      break;
+    } else if (inParens) {
+      if (*iter == char16_t(')')) {
+        inParens = false;
+      }
+    } else {
+      if (*iter == char16_t(',')) {
+        // End of descriptors, flush current descriptor and advance past comma
+        // before breaking
+        descriptors.AddDescriptor(Substring(currentDescriptor, iter));
+        iter++;
+        break;
+      } else if (nsContentUtils::IsHTMLWhitespace(*iter)) {
+        // End of current descriptor, consume it, skip spaces
+        // ("After descriptor" state in spec) before continuing
+        descriptors.AddDescriptor(Substring(currentDescriptor, iter));
+        for (; iter != end && nsContentUtils::IsHTMLWhitespace(*iter); ++iter);
+        if (iter == end) {
+          break;
+        }
+        currentDescriptor = iter;
+        // Leave one whitespace so the loop advances to this position next iteration
+        iter--;
+      } else if (*iter == char16_t('(')) {
+        inParens = true;
+      }
+    }
   }
 
-  return true;
+  descriptors.FillCandidate(*this);
+
+  return Type() != eCandidateType_Invalid;
 }
 
 bool
@@ -533,11 +666,10 @@ ResponsiveImageCandidate::HasSameParameter(const ResponsiveImageCandidate & aOth
   return false;
 }
 
-already_AddRefed<nsIURI>
-ResponsiveImageCandidate::URL() const
+const nsAString&
+ResponsiveImageCandidate::URLString() const
 {
-  nsCOMPtr<nsIURI> url = mURL;
-  return url.forget();
+  return mURLString;
 }
 
 double
